use askama_axum::IntoResponse;
use axum::extract::{Path, State};
use axum::http::StatusCode;
use log::error;

use crate::entities::{prelude::*, *};
use crate::AppState;
use sea_orm::*;

use crate::block_types::{generate_content_blocks, GenericBlock};
use chrono::{Datelike, Utc};

use super::{PageNotFoundTemplate, PageTemplate};

pub(crate) async fn resolver(
    Path(slug): Path<String>,
    state: State<AppState>,
) -> impl IntoResponse {
    let menu: Vec<menu_entry::Model> = match Menu::find()
        .filter(menu::Column::Name.eq("primary"))
        .one(&state.db_conn)
        .await
        .expect("Failed to get primary menu! HALTING")
    {
        Some(menu) => menu,
        None => panic!("Missing \"primary\" menu in database!"),
    }
    .find_related(MenuEntry)
    .order_by_asc(menu_entry::Column::Order)
    .all(&state.db_conn)
    .await
    .expect("Failed to get primary menu items!");

    let settings: settings::Model = match Settings::find()
        .filter(settings::Column::Id.eq("current"))
        .one(&state.db_conn)
        .await
        .expect("Failed to get site settings!")
    {
        Some(settings) => settings,
        None => panic!("Missing \"current\" settings in database!"),
    };

    let page_search: Result<Option<page::Model>, DbErr> = Page::find()
        .filter(page::Column::Slug.eq(&slug))
        .one(&state.db_conn)
        .await;

    let page_meta: page::Model = match page_search {
        Ok(page_result) => match page_result {
            Some(page_result) => page_result,
            None => {
                return (
                    StatusCode::NOT_FOUND,
                    PageNotFoundTemplate {
                        slug,
                        year: Utc::now().date_naive().year().to_string(),
                        menu,
                        settings,
                    },
                )
                    .into_response()
            }
        },
        Err(e) => {
            error!("{}", e);
            return (StatusCode::INTERNAL_SERVER_ERROR, ()).into_response();
        }
    };

    let blocks: Vec<GenericBlock> = page_meta
        .find_related(PageBlock)
        .order_by_asc(page_block::Column::Order)
        .all(&state.db_conn)
        .await
        .expect("Failed to get home page blocks! HALTING")
        .into_iter()
        .map(|f| GenericBlock {
            id: f.id,
            owner: f.owner,
            order: f.order,
            block_type: f.r#type,
            content: f.content,
        })
        .collect();

    let content_blocks = generate_content_blocks(&state.db_conn, blocks).await;

    (
        StatusCode::OK,
        PageTemplate {
            draft: page_meta.clone().draft,
            title: page_meta.clone().title,
            description: page_meta.clone().description,
            show_title: page_meta.clone().show_title,
            content_blocks,
            year: Utc::now().date_naive().year().to_string(),
            menu,
            settings,
        },
    )
        .into_response()
}
