use askama_axum::IntoResponse;
use axum::extract::{Path, State};
use axum::http::StatusCode;
use log::error;

use crate::entities::{prelude::*, *};
use crate::AppState;
use sea_orm::*;

use crate::block_types::{generate_content_blocks, GenericBlock};
use chrono::{Datelike, Utc};

use super::{PostNotFoundTemplate, PostTemplate};

pub(crate) async fn resolver(
    Path(slug): Path<String>,
    state: State<AppState>,
) -> impl IntoResponse {
    let menu: Vec<menu_entry::Model> = match Menu::find()
        .filter(menu::Column::Name.eq("primary"))
        .one(&state.db_conn)
        .await
        .expect("Failed to get primary menu! HALTING")
    {
        Some(menu) => menu,
        None => panic!("Missing \"primary\" menu in database!"),
    }
    .find_related(MenuEntry)
    .order_by_asc(menu_entry::Column::Order)
    .all(&state.db_conn)
    .await
    .expect("Failed to get primary menu items!");

    let settings: settings::Model = match Settings::find()
        .filter(settings::Column::Id.eq("current"))
        .one(&state.db_conn)
        .await
        .expect("Failed to get site settings!")
    {
        Some(settings) => settings,
        None => panic!("Missing \"current\" settings in database!"),
    };

    let post_search: Result<Option<post::Model>, DbErr> = Post::find()
        .filter(post::Column::Slug.eq(&slug))
        .one(&state.db_conn)
        .await;

    let post_meta: post::Model = match post_search {
        Ok(post_result) => match post_result {
            Some(post_result) => post_result,
            None => {
                return (
                    StatusCode::NOT_FOUND,
                    PostNotFoundTemplate {
                        slug,
                        year: Utc::now().date_naive().year().to_string(),
                        menu,
                        settings,
                    },
                )
                    .into_response()
            }
        },
        Err(e) => {
            error!("{}", e);
            return (StatusCode::INTERNAL_SERVER_ERROR, ()).into_response();
        }
    };

    let blocks: Vec<GenericBlock> = post_meta
        .find_related(PostBlock)
        .order_by_asc(post_block::Column::Order)
        .all(&state.db_conn)
        .await
        .expect("Failed to get post blocks! HALTING")
        .into_iter()
        .map(|f| GenericBlock {
            id: f.id,
            owner: f.owner,
            order: f.order,
            block_type: f.r#type,
            content: f.content,
        })
        .collect();

    let content_blocks = generate_content_blocks(&state.db_conn, blocks).await;

    let last_updated = post_meta.clone().updated.map(|updated| updated.to_string());

    let featured_image: Option<image::Model> = match post_meta.clone().featured_image {
        Some(featured_image) => Image::find()
            .filter(image::Column::Id.eq(featured_image))
            .one(&state.db_conn)
            .await
            .expect("Failed to get image! HALTING"),
        None => None,
    };

    (
        StatusCode::OK,
        PostTemplate {
            draft: post_meta.clone().draft,
            title: post_meta.clone().title,
            summary: post_meta.clone().summary,
            publish_date: post_meta.clone().published.to_string(),
            last_updated,
            content_blocks,
            year: Utc::now().date_naive().year().to_string(),
            menu,
            settings,
            featured_image,
        },
    )
        .into_response()
}
